@@ -1,7 +1,11 @@
 from .registration.generate_registration_options import generate_registration_options
 from .registration.verify_registration_response import verify_registration_response
-from .authentication.generate_authentication_options import generate_authentication_options
-from .authentication.verify_authentication_response import verify_authentication_response
+from .authentication.generate_authentication_options import (
+    generate_authentication_options,
+)
+from .authentication.verify_authentication_response import (
+    verify_authentication_response,
+)
 from .helpers import base64url_to_bytes, options_to_json
 
-__version__ = "1.1.0"
+__version__ = "1.2.0"
